@@ -1,6 +1,5 @@
 import socket
 import os
-
 import gymnasium as gym
 import math
 import numpy as np
@@ -11,6 +10,7 @@ from FlagCapture.miniworld import MiniWorldEnv
 from FlagCapture.utils import get_file_path
 from  FlagCapture.build import BUILD
 from threading import Thread as T
+import sys
 
 BUILD()
 
@@ -144,7 +144,8 @@ scene.agent.dir = myinfo['dir']
 scene.render()
 scene.unwrapped.window.push_handlers(board)
 scene.unwrapped.window.set_icon(pyglet.image.load(get_file_path("images", "icon", "png")))
-scene.unwrapped.window.set_caption(f'Capture the Flag [{myTeam} team]')
+caption = f'Capture the Flag: [{myTeam} Team]'
+scene.unwrapped.window.set_caption(caption)
 scene.unwrapped.window.maximize()
 scene.unwrapped.window.set_mouse_visible(False)
 
@@ -207,10 +208,12 @@ def update(dt):
     if board[key.P]:
         if scene.intersect(scene.agent, scene.agent.pos, scene.agent.radius) == scene.redFlag:
             carrying = 'red'
+            scene.unwrapped.window.set_caption(f'{caption} [CARRYING A FLAG]')
             cl.send(str(['Updating carrying', {'carrying': 'red'}]).encode('utf-8'))
             scene.redFlag.pos = [30, -90, 0]
         if scene.intersect(scene.agent, scene.agent.pos, scene.agent.radius) == scene.blueFlag:
             carrying = 'blue'
+            scene.unwrapped.window.set_caption(f'{caption} [CARRYING A FLAG]')
             cl.send(str(['Updating carrying', {'carrying': 'blue'}]).encode('utf-8'))
             scene.blueFlag.pos = [30, -90, 0]
 
@@ -237,6 +240,8 @@ def red_wins():
     pyglet.clock.schedule_once(close, 4)
     alert(get_file_path("images", "lynxwin", "png"), 4)
 
+creating = False
+
 def drop_flag():
     global carrying
     if carrying:
@@ -248,6 +253,7 @@ def drop_flag():
             if scene.blueFlag.pos[2] > 0:red_wins()
         cl.send(str(['drop', {'flag': carrying}]).encode('utf-8'))
         carrying = None
+        scene.unwrapped.window.set_caption(caption)
 
 carrying = None
 
@@ -274,21 +280,25 @@ def recv():
 
 raw_message = None
 
+message = {}
+
 def process_message(dt):
-    global IAmPrisoner
+    global IAmPrisoner, message, creating
     try:
         message = eval(raw_message)
     except SyntaxError:
         return
     for person in message:
-        if person not in people and person != myId:#Some one joined
-            animal = scene.place_entity(MeshEnt(mesh_name=f''+message[person]['team'], height=1.4, static=False), pos=message[person]['pos'], dir=message[person]['dir'])
+        if person not in people and person != myId and not creating:#Some one joined
+            creating = True
+            animal = scene.place_entity(MeshEnt(mesh_name=message[person]['team'], height=1.4, static=False), pos=message[person]['pos'], dir=message[person]['dir'])
             print(message[person]['pos'])
             people[person] = animal
             if message[person]['team'] == myTeam:
                 list_of_players_on_my_team.append(person)
             else:
                 list_of_players_on_their_team.append(person)
+            creating = False
 
         elif person != myId: #Update the players
             people[person].pos = message[person]['pos']
@@ -296,17 +306,26 @@ def process_message(dt):
             if message[person]['carrying'] == 'red':
                 scene.redFlag.pos = [message[person]['pos'][0], 0, message[person]['pos'][2]]
                 if scene.redFlag.pos[2] < 0:blue_wins()
-                scene.redFlag.dir = message[person]['dir']
+                scene.redFlag.dir = math.radians(math.degrees(message[person]['dir'])+180)
             if message[person]['carrying'] == 'blue':
                 scene.blueFlag.pos = [message[person]['pos'][0], 0, message[person]['pos'][2]]
                 if scene.blueFlag.pos[2] > 0:red_wins()
-                scene.blueFlag.dir = message[person]['dir']
+                scene.blueFlag.dir = math.radians(math.degrees(message[person]['dir'])+180)
 
             check_for_capture_and_touching_button()
 
             if person in list_of_players_on_my_team and IAmPrisoner and message[person]['touching']:#Check to see if someone is freeing you
                     IAmPrisoner=False
                     alert(get_file_path("images", "release", "png"), 2.5)
+    for person in people:
+        if person not in message:#Some one left
+            scene.unwrapped.entities.remove(people[person])
+            del people[person]
+            break
+
+@scene.unwrapped.window.event
+def on_close():
+    cl.close()
 
 touching = False
 
@@ -328,18 +347,40 @@ def check_for_capture_and_touching_button():
     #Check for touching enemy and in enemy territory
     for person in list_of_players_on_their_team:
         if intersection == people[person] and scene.agent.pos[2] < 0:
+            if myTeam == 'wolf':
+                if carrying:
+                    drop_flag()
+                    scene.agent.pos = [48, 0, -55]
+                    scene.agent.dir = 4.754
+                    IAmPrisoner = True
+                    cl.send(str(['Updating prisoner', {'prisoner': [48, 0, -55]}]).encode('utf-8'))
+                    cl.send(str(['Updating pos', {'pos': [48, 0, -55]}]).encode('utf-8'))
             if myTeam == 'lynx':
+                if message[person]['carrying']: return
+                if message[person]['prisoner']: return
                 drop_flag()
                 scene.agent.pos = [48, 0, 55]
                 scene.agent.dir = 1.654
                 IAmPrisoner = True
+                cl.send(str(['Updating prisoner', {'prisoner': [48, 0, -55]}]).encode('utf-8'))
                 cl.send(str(['Updating pos', {'pos': [48, 0, 55]}]).encode('utf-8'))
         if intersection == people[person] and scene.agent.pos[2] > 0:
+            if myTeam == 'lynx':
+                if carrying:
+                    drop_flag()
+                    scene.agent.pos = [48, 0, 55]
+                    scene.agent.dir = 1.654
+                    IAmPrisoner = True
+                    cl.send(str(['Updating prisoner', {'prisoner': [48, 0, -55]}]).encode('utf-8'))
+                    cl.send(str(['Updating pos', {'pos': [48, 0, 55]}]).encode('utf-8'))
             if myTeam == 'wolf':
+                if message[person]['carrying']:return
+                if message[person]['prisoner']: return
                 drop_flag()
                 scene.agent.pos = [48, 0, -55]
                 scene.agent.dir = 4.754
                 IAmPrisoner = True
+                cl.send(str(['Updating prisoner', {'prisoner': [48, 0, -55]}]).encode('utf-8'))
                 cl.send(str(['Updating pos', {'pos': [48, 0, -55]}]).encode('utf-8'))
 
 
